package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.ParamName;
import com.ether.user.User;
import com.ether.user.UserFilter;
import com.ether.user.UserRole;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;

import java.security.DigestException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author vmipsl
 * @date 13 nov 2011
 */
public class ControllerBackoffice
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(view = VIEW_BO_INDEX, backofficeMethod = true, defaultView = VIEW_BO_INDEX)
    public Map<String, Object> home()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_BO_USER, backofficeMethod = true, defaultView = VIEW_BO_INDEX, jsonResult = true)
    public JSONObject viewUser()
            throws ServiceException, WebException
    {
        final JSONObject jsonObject = sortUser( "lastName", null, "ALL", true, true, 5, 1 );
        jsonObject.put( "jSonUserRoles", getJSONUserRoles() );
        return jsonObject;
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(jsonResult = true)
    public JSONObject addUser( @NotNull @ParamName(ParameterConstants.PARAMETER_NAME) final String lastName,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_FIRST_NAME) final String firstName,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_EMAIL) final String email,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_PWD) final String pwd,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_ROLE) final String role,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_LABORATORY) final String laboratory,
                               @Nullable @ParamName(ParameterConstants.PARAMETER_COUNTRY) final String country,
                               @NotNull @ParamName(ParameterConstants.PARAMETER_HAS_ACCESS) final Boolean hasAccessToBO )
            throws WebException, ServiceException
    {
        try
        {
            final User existingUser = _backofficeService.getUserByEmail( email );
            if( null == existingUser )
            {
                final Date creationDate = new Date();
                final String encryptedPassword = EtherHelper.encryptPassword( pwd );
                final User user = new User( lastName, firstName, email, encryptedPassword, role, hasAccessToBO, laboratory, country, creationDate );

                _backofficeService.createUser( user );
            }
            else
                throw new WebException( WebException.WebCode.USER_ALREADY_EXISTS, "This email already corresponds to an User", WebException.getExceptionThrowable() );

        }
        catch( DigestException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }
        catch( NoSuchAlgorithmException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }

        return getAllUsers();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject removeUser( @NotNull @ParamName(ParameterConstants.PARAMETER_ID) final Integer userId )
            throws WebException, ServiceException
    {
        _backofficeService.removeUserById( userId );

        return getAllUsers();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject modifyUser( @NotNull @ParamName(ParameterConstants.PARAMETER_ID) final Integer userId,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_NAME) final String lastName,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_FIRST_NAME) final String firstName,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_EMAIL) final String email,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_PWD) final String pwd,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_ROLE) final String role,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_LABORATORY) final String laboratory,
                                  @Nullable @ParamName(ParameterConstants.PARAMETER_COUNTRY) final String country,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_HAS_ACCESS) final Boolean hasAccessToBO,
                                  @NotNull @ParamName(ParameterConstants.PARAMETER_KEEP_SAME_PASSWORD) final Boolean keepSamePassword )
            throws WebException, ServiceException
    {
        try
        {
            final User existingUser = _backofficeService.getUserByEmail( email );
            if( null == existingUser || userId.equals( existingUser.getId() ) )
            {
                final User user = _backofficeService.getUserById( userId );
                user.setLastName( lastName );
                user.setFirstName( firstName );
                user.setEmail( email );
                if( !keepSamePassword && null != pwd )
                {
                    final String encryptedPassword = EtherHelper.encryptPassword( pwd );
                    user.setPassword( encryptedPassword );
                }
                user.setRole( UserRole.valueOf( role ) );
                user.setAccessToBO( hasAccessToBO );
                user.setLaboratory( laboratory );
                user.setCountry( country );

                _backofficeService.updateUser( user );
            }
            else
                throw new WebException( WebException.WebCode.USER_ALREADY_EXISTS, "This email already corresponds to an User", WebException.getExceptionThrowable() );
        }
        catch( DigestException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }
        catch( NoSuchAlgorithmException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }

        return getAllUsers();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject sortUser( @NotNull @ParamName("sort") final String sort,
                                @Nullable @ParamName("searchText") final String searchText,
                                @NotNull @ParamName("sortRole") final String sortRole,
                                @NotNull @ParamName("sortAccessBoTrue") final Boolean sortAccessBoTrue,
                                @NotNull @ParamName("sortAccessBoFalse") final Boolean sortAccessBoFalse,
                                @ParamName(ParameterConstants.PARAMETER_MAX_RESULTS) final Integer maxResults,
                                @ParamName(ParameterConstants.PARAMETER_PAGE) final Integer page )
            throws WebException, ServiceException
    {
        final UserFilter filter = new UserFilter( sort, searchText, sortAccessBoFalse, sortAccessBoTrue, maxResults, page );
        if( !"ALL".equals( sortRole ) )
            filter.setSortRole( UserRole.valueOf( sortRole ) );

        final PaginatedResult<User> users = _backofficeService.searchUsers( filter );

        final JSONObject result = new JSONObject();
        result.put( "jSonUsers", getJsonHelper().toJSON( users.getPaginatedResults() ) );
        result.put( "usersNumber", users.getTotalCount() );
        return result;
    }

    private JSONObject getAllUsers()
            throws ServiceException
    {
        final List<User> users = _backofficeService.getAllUsersByNameOrder();

        final JSONObject result = new JSONObject();
        result.put( "jSonUsers", getJsonHelper().toJSON( users ) );
        return result;
    }

    private List<JSONObject> getJSONUserRoles()
    {
        final UserRole[] userRoles = UserRole.values();

        final List<JSONObject> jsonUserRoles = new ArrayList<JSONObject>( userRoles.length );

        for( final UserRole userRole : userRoles )
        {
            final JSONObject jsonUserRole = new JSONObject();
            jsonUserRole.put( "text", userRole.name() );
            jsonUserRole.put( "value", userRole.name() );
            jsonUserRoles.add( jsonUserRole );
        }
        return jsonUserRoles;
    }

    @Required
    public void setBackofficeService( final BackofficeService backofficeService )
    {
        _backofficeService = backofficeService;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerBackoffice.class );

    private static final String VIEW_BO_INDEX = "backoffice/index";
    private static final String VIEW_BO_USER = "backoffice/user";

    private BackofficeService _backofficeService;
}
